package IkiWiki::Plugin::hl;

use warnings;
use strict;
use IkiWiki 2.00;

use File::Temp;

our $VERSION = '0.02';

our $exec_cmd = q(/usr/bin/highlight --quiet --force --xhtml --fragment --linenumbers);

sub import {
    hook ( type => "preprocess", id => 'hl', call => \&preprocess );
}

=pod

In a ikiwiki source page we found 

[[!hl syntax="ini" source="""

# This is my first config file 
root = /tmp

"""]] 

=cut

sub preprocess {
    my  %params =   @_;
    my  $html   = '';

    # clean the parameters
    $params{syntax} =~ s/[^\w]+//g;
    
    # get the source 
    my $fh = File::Temp->new();

    $fh->print( $params{source} );

    $fh->close();

    # run the highlight program 
    $html = _run_highlight( $fh->filename(), $params{syntax} );

    # clean the html output
    1;

    return $html;
}

sub _run_highlight {
    my  ( $input_file, $syntax ) = @_;

    my  $cmd = sprintf('%s --syntax %s < %s ', $exec_cmd, $syntax, $input_file );
    my  $ret = '';

    if (open(HL, "$cmd |")) {
        while (<HL>) {
            $ret .= $_;
        }
        close HL;
    }
    else {
        die "could not execute $cmd - $!";
    }

    return $ret;
}

1; 
